import {Component, OnInit} from "@angular/core";

@Component({
    selector: 'plugin-inbox-admin',
    templateUrl: 'plugin-inbox-admin.component.html',
    styleUrls: ['plugin-inbox-admin.component.css']
})
export class PluginInboxAdminComponent  implements OnInit {

    ngOnInit() {

    }
}
